<?php
/*
Plugin Name: Apple Music Now Playing API
Description: Accepts JSON now-playing updates via REST, lets user set a secure API key on the plugin page, a shortcode widget, that can be added to Footer
Version: 1.4.1
Author: Sophia Atkinson
*/

defined("ABSPATH") || exit();

// === Register REST API endpoints ===
add_action("rest_api_init", function () {
    register_rest_route("nowplaying/v1", "/update", [
        "methods" => "POST",
        "callback" => "nowplaying_api_handle_post",
        "permission_callback" => "__return_true",
    ]);
    register_rest_route("nowplaying/v1", "/update/artwork", [
        "methods" => "POST",
        "callback" => "nowplaying_api_handle_art_post",
        "permission_callback" => "__return_true",
    ]);
});

function nowplaying_api_handle_post(WP_REST_Request $request)
{
    $stored_key = get_option("nowplaying_api_key");
    $auth_header = $request->get_header("authorization");

    $expected_prefix = "bearer ";
    $auth_header_normalized = strtolower(trim($auth_header ?? ""));
    if (
        !$auth_header ||
        stripos($auth_header_normalized, $expected_prefix) !== 0 ||
        substr(trim($auth_header), strlen($expected_prefix)) !== $stored_key
    ) {
        return new WP_REST_Response("Forbidden: Invalid API key.", 403);
    }

    $data = json_decode($request->get_body(), true);

    if ($data === null) {
        return new WP_REST_Response("Invalid JSON.", 400);
    }

    $file_path = plugin_dir_path(__FILE__) . "nowplaying.json";
    $written = file_put_contents(
        $file_path,
        json_encode($data, JSON_PRETTY_PRINT)
    );
    try {
        $written = file_put_contents(
            $file_path,
            json_encode($data, JSON_PRETTY_PRINT)
        );
        if ($written === false) {
            throw new Exception();
        }
    } catch (Exception $e) {
        return new WP_REST_Response("Failed to write to nowplaying.json.", 500);
    }

    return new WP_REST_Response("Now Playing updated.", 200);
}

function nowplaying_api_handle_art_post(WP_REST_Request $request)
{
    $stored_key = get_option("nowplaying_api_key");
    $auth_header = $request->get_header("authorization");

    if (!$auth_header || $auth_header !== "Bearer $stored_key") {
        return new WP_REST_Response("Forbidden: Invalid API key.", 403);
    }

    $data = json_decode($request->get_body(), true);

    if ($data === null) {
        return new WP_REST_Response("Invalid JSON.", 400);
    }

    if (!isset($data["image"]) || !is_string($data["image"])) {
        return new WP_REST_Response(
            'Missing or invalid "image" property.',
            400
        );
    }

    $file_path = plugin_dir_path(__FILE__) . "nowplaying-art.json";

    try {
        $written = file_put_contents(
            $file_path,
            json_encode($data, JSON_PRETTY_PRINT)
        );
        if ($written === false) {
            throw new Exception();
        }
    } catch (Exception $e) {
        return new WP_REST_Response(
            "Failed to write to nowplaying-art.json.",
            500
        );
    }

    return new WP_REST_Response("Artwork updated.", 200);
}

// === Admin Settings Page ===
add_action("admin_menu", function () {
    add_options_page(
        "Now Playing API Settings",
        "Now Playing API",
        "manage_options",
        "nowplaying-api",
        "nowplaying_api_settings_page"
    );
});

function nowplaying_api_settings_page()
{
    if (!current_user_can("manage_options")) {
        return;
    }

    if (
        isset($_POST["nowplaying_api_key"]) &&
        check_admin_referer("nowplaying_save_key", "nowplaying_nonce")
    ) {
        $new_key = sanitize_text_field($_POST["nowplaying_api_key"]);
        if (strlen($new_key) === 64) {
            update_option("nowplaying_api_key", $new_key);
            echo '<div class="updated"><p>API key updated.</p></div>';
        } else {
            echo '<div class="error"><p>API key must be exactly 64 characters.</p></div>';
        }
    }

    if (
        isset($_POST["generate_key"]) &&
        check_admin_referer("nowplaying_save_key", "nowplaying_nonce")
    ) {
        $generated_key = bin2hex(random_bytes(32));
        update_option("nowplaying_api_key", $generated_key);
        echo '<div class="updated"><p>New API key generated.</p></div>';
    }

    $current_key = get_option("nowplaying_api_key");
    if (!$current_key) {
        $current_key = bin2hex(random_bytes(32));
        update_option("nowplaying_api_key", $current_key);
    }

    $url = esc_url(site_url("/wp-json/nowplaying/v1/update"));
    $artwork_url = esc_url(site_url("/wp-json/nowplaying/v1/update/artwork"));
    ?>
    <div class="wrap">
        <h1>Apple Music Now Playing API Settings</h1>
        <p>This is the key you will use for the web extension to communicate with this server. The POST addresses are:</p>
        Now playing Endpoint: <code><?php echo $url; ?></code>
        <br />
        Artwork Endpoint: <code><?php echo $artwork_url; ?></code>

        <form method="post">
            <?php wp_nonce_field("nowplaying_save_key", "nowplaying_nonce"); ?>
            <table class="form-table">
                <tr>
                    <th><label for="nowplaying_api_key">Current API Key</label></th>
                    <td>
                        <input type="text" name="nowplaying_api_key" value="<?php echo esc_attr(
                            $current_key
                        ); ?>" class="regular-text" maxlength="64" />
                        <p class="description">Keep this key secret. <br> Must be 64 characters.</p>
                    </td>
                </tr>
            </table>
            <?php submit_button("Save API Key"); ?>
        </form>

        <form method="post" style="margin-top: 20px;">
            <?php wp_nonce_field("nowplaying_save_key", "nowplaying_nonce"); ?>
            <?php submit_button(
                "Generate New API Key",
                "secondary",
                "generate_key"
            ); ?>
        </form>
    </div>
    <?php
}

// === Shortcode [now-playing-widget] ===
add_shortcode("now-playing-widget", function () {
    $plugin_url = plugin_dir_url(__FILE__) . "nowplaying.json";
    $art_url = plugin_dir_url(__FILE__) . "nowplaying-art.json";
    ob_start();
    ?>
<?php $placeholder_image = 'data:image/svg+xml;base64,' . base64_encode(file_get_contents('https://placehold.co/150/1E1F22/FFF?text=Nothing%20playing')); ?>
<?php $css_url = plugin_dir_url(__FILE__) . "assets/css/amnp-style.css"; ?>
<link rel="stylesheet" type="text/css" href="<?php echo esc_url($css_url); ?>">
    <div class="amnp hasjs" id="widget">
      <img title="" id="cover" src="<?php echo $placeholder_image; ?>" alt="Album cover" />
      <div title="" id="title">Loading...</div>
      <div title="" id="artist"></div>
      <div id="status"></div>
      <div id="progress-container"><div id="progress-bar"></div></div>
      <div id="time-info"><span id="current-time">0:00</span><span id="total-time">0:00</span></div>
    </div>
 <?php
 $json_path = plugin_dir_path(__FILE__) . "nowplaying.json";
 $art_path = plugin_dir_path(__FILE__) . "nowplaying-art.json";
 $nowplaying_data = @json_decode(file_get_contents($json_path), true) ?? [];
 $art_data = @json_decode(file_get_contents($art_path), true) ?? [];
 $np_details = esc_html($nowplaying_data["details"] ?? "Not playing");
 $np_artist = esc_html($nowplaying_data["artist"] ?? "");
 $np_status = isset($nowplaying_data["paused"]) && $nowplaying_data["paused"]  ? "Paused" : "Playing";
 $np_cover = esc_attr($art_data["image"] ?? ($nowplaying_data["artworkBase64"] ?? ""));
 $np_album = esc_attr($nowplaying_data["album"] ?? "");
 ?>
    <noscript>
  <div class="amnp" id="widget">
    <img src="<?php echo $np_cover ?: $placeholder_image; ?>" alt="Album cover" title="<?php echo $np_album; ?>" />
    <div title="<?php echo $np_details; ?>"><?php echo $np_details; ?></div>
    <div title="<?php echo $np_artist; ?>"><?php echo $np_artist; ?></div>
    <div><?php echo $np_status; ?></div>
  </div>
</noscript>

    <script>
    let lastNowPlaying = null;
let lastSongKey = null;
let lastArtImage = null;
let intervalId = null;
function formatTime(s){if(!s||isNaN(s)){return "0:00"}let m=Math.floor(s/60),sec=Math.floor(s%60);return m+":"+(sec<10?"0":"")+sec}
async function fetchJson(url) {
    try {
        const res = await fetch(url + '?' + Date.now(), { cache: 'no-store' });
        if (!res.ok) throw new Error(`Fetch failed: ${res.status}`);
        return await res.json();
    } catch (e) {
        console.error('Fetch error:', e);
        return null;
    }
}
async function updateWidget() {
    if (document.hidden) return;
    const nowPlaying = await fetchJson('<?php echo esc_url($plugin_url); ?>');
    if (!nowPlaying) return;
    const currentSongKey = (nowPlaying.details || '') + '|' + (nowPlaying.album || '');
    if (JSON.stringify(nowPlaying) !== JSON.stringify(lastNowPlaying)) {
        lastNowPlaying = nowPlaying;

        document.getElementById('title').textContent = nowPlaying.details || 'Not playing';
        document.getElementById('title').title = nowPlaying.details || '';
        document.getElementById('artist').textContent = nowPlaying.state || '';
        document.getElementById('artist').title = nowPlaying.state || '';
        document.getElementById('status').textContent = nowPlaying.paused ? 'Paused' : 'Playing';
        const currentTime = typeof nowPlaying.currentTime === 'number' ? nowPlaying.currentTime : 0;
        const duration = typeof nowPlaying.duration === 'number' ? nowPlaying.duration : 0;
        document.getElementById('progress-bar').style.width =
            (duration > 0 ? (currentTime / duration) * 100 : 0) + '%';
        document.getElementById('current-time').textContent = formatTime(nowPlaying.currentTime);
        document.getElementById('total-time').textContent = formatTime(nowPlaying.duration);
    }
    if (currentSongKey !== lastSongKey) {
        lastSongKey = currentSongKey;
        const artData = await fetchJson('<?php echo esc_url($art_url); ?>');
        let image = null;
        if (artData?.image) {
            image = artData.image;
        } else if (nowPlaying.artworkBase64) {
            image = nowPlaying.artworkBase64;
        }
        if (image && image !== lastArtImage) {
            lastArtImage = image;
            const cover = document.getElementById('cover');
            cover.src = image;
            cover.alt = nowPlaying.details || 'Album cover';
            cover.title = nowPlaying.album || '';
        }
    }
}
function startLoop() {
    if (intervalId === null) {
        updateWidget();
        intervalId = setInterval(updateWidget, 1000);
    }
}
function stopLoop(){if (intervalId !== null) {clearInterval(intervalId);intervalId = null;}}
const widget = document.getElementById('widget');
let isVisible = false;
const observer = new IntersectionObserver((entries) => {
    entries.forEach(entry => {isVisible = entry.isIntersecting;updateFetchState();});
}, { threshold: 0.1 });

function updateFetchState() {
    if (!document.hidden && isVisible) {
        startLoop();
    } else {
        stopLoop();
    }
}
observer.observe(widget);
document.addEventListener('visibilitychange', updateFetchState);
updateFetchState();
    </script>
    <?php return ob_get_clean();
}); // === Register Customizer Widget Support with Title Option ===
class Now_Playing_Widget extends WP_Widget
{
    public function __construct()
    {
        parent::__construct(
            "now_playing_widget",
            __("Now Playing Widget", "nowplaying"),
            [
                "description" => __(
                    "Displays the current Apple Music song from your API.",
                    "nowplaying"
                ),
            ]
        );
    }
    public function widget($args, $instance)
    {
        $title = apply_filters("widget_title", $instance["title"] ?? "");
        echo $args["before_widget"];
        if (!empty($title)) {
            echo $args["before_title"] .
                esc_html($title) .
                $args["after_title"];
        }
        echo do_shortcode("[now-playing-widget]");
        echo $args["after_widget"];
    }
    public function form($instance)
    {
        $title = esc_attr($instance["title"] ?? ""); ?>
        <p>
            <label for="<?php echo esc_attr(
                $this->get_field_id("title")
            ); ?>"><?php _e("Title:"); ?></label>
            <input class="widefat" id="<?php echo esc_attr(
                $this->get_field_id("title")
            ); ?>"
                   name="<?php echo esc_attr(
                       $this->get_field_name("title")
                   ); ?>" type="text"
                   value="<?php echo $title; ?>" />
        </p>
        <?php
    }
    public function update($new_instance, $old_instance)
    {
        $instance = [];
        $instance["title"] = sanitize_text_field($new_instance["title"] ?? "");
        return $instance;
    }
}
add_action("widgets_init", function () {
    register_widget("Now_Playing_Widget");
});
